package main

import (
	"database/sql"
	"fmt"
	"html/template"
	"log"
	"net/http"
	"os"
	"strings"

	_ "github.com/mattn/go-sqlite3"
)

var db *sql.DB


type Todo struct {
  Title string 
  Desc string
  Done bool
}


func get_requestee(requestee string) string {
  var addr string
  var addr_list []string

  if strings.Contains(requestee, "[") {
    addr_list = strings.Split(requestee, ":")
    addr = strings.Join(addr_list[1:len(addr_list)-1], "")
    addr = "[::" + addr

  } else {
    addr_list = strings.Split(requestee, ":")
    addr = strings.Join(addr_list[:len(addr_list)-1], "")
  }

  return addr
}

func init_db() {
  os.Remove("./squeal.db")

  database, err := sql.Open("sqlite3", "./squeal.db")
  if err != nil {
    log.Fatal(err)
  }
  db = database

  table := `create table todos (title text not null, description text, done integer)`
  _, err = db.Exec(table)
  if err != nil {
    log.Fatal(err)
  }
}

func todo_add(title string, description string, done bool) {
  done_int := 0
  if done {
    done_int = 1
  }
  stmt := fmt.Sprintf("insert into todos (title, description, done) values (\"%s\", \"%s\", \"%d\")", title, description, done_int)

  _, err := db.Exec(stmt)
  if err != nil {
    log.Print(err)
  }
}

func todo_get_all() []Todo {
  stmt := `select * from todos`
  row, err := db.Query(stmt)
  if err != nil {
    log.Print(err)
  }

  res := []Todo{}
  for row.Next() {
    var todo Todo
    var done_tmp int

    row.Scan(&todo.Title, &todo.Desc, &done_tmp)

    todo.Done = false
    if done_tmp != 0 {
      todo.Done = true
    }

    res = append(res, todo)
  }

  return res
}

func pending(w http.ResponseWriter, r *http.Request) {
  log.Print(get_requestee(r.RemoteAddr) + " " + r.Method + " pending")

  todos := map[string][]Todo{}
  for _, todo := range todo_get_all() {
    if todo.Done == false {
      todos["Todos"] = append(todos["Todos"], todo)
    }
  }

  tmpl := template.Must(template.ParseFiles("tmpl/home.html"))
  tmpl.Execute(w, todos)
}

func home(w http.ResponseWriter, r *http.Request) {
  log.Print(get_requestee(r.RemoteAddr) + " " + r.Method + " home")
  
  todos := map[string][]Todo{}
  for _, todo := range todo_get_all() {
    todos["Todos"] = append(todos["Todos"], todo)
  }

  tmpl := template.Must(template.ParseFiles("tmpl/home.html"))
  tmpl.Execute(w, todos)
}

func add_todo(w http.ResponseWriter, r *http.Request) {
  log.Print(get_requestee(r.RemoteAddr) + " " + r.Method + " add-todo")

  r.ParseForm()
  title := r.Form.Get("title")
  desc := r.Form.Get("desc")
  done := strings.Contains(r.Form.Get("selected"), "done")

  todo_add(title, desc, done)

  todo := Todo{title, desc, done}

  tmpl := template.Must(template.ParseFiles("tmpl/home.html"))
  tmpl.ExecuteTemplate(w, "todo-table-element", todo)
}


func main() {
  init_db()

  http.HandleFunc("/", home)
  http.HandleFunc("/add-todo", add_todo)
  http.HandleFunc("/pending", pending)

  log.Print("Listening")
  log.Fatal(http.ListenAndServe("localhost:8000", nil))
}
